import { visualList } from './visualList';

const fileExtension = str => str.split('.').pop();
const colorValue = d => fileExtension(d.data.name);

const width = 960;
const height = 500;
const xOffset = 100;
const yOffset = -10;
const svg = d3.select('body').append('svg')
  .attr('width', width)
  .attr('height', height);
const g = svg.append('g');
const tree = d3.tree().size([Math.PI, height]);

const colorScale = d3.scaleOrdinal(
  d3.schemeCategory10
    .concat(d3.range(200).map(() => d3.schemeCategory10[9])) // Don't loop the colors.
);

svg
  .append('rect')
    .attr('width', width)
    .attr('height', height)
    .style('fill', 'none')
    .style('pointer-events', 'all')
    .call(d3.zoom()
    .on('zoom', zoomed));

function zoomed() {
  g.attr('transform', d3.event.transform);
}
var colorLegend = d3.legendColor().shape('circle');
const commaFormat = d3.format(',');

d3.json('https://vizhub.com/curran/datasets/git-source-tree-.csv', function (data){

  var hierarchy = d3.hierarchy(data[0], d => d.contents);
  tree(hierarchy);
  var descendants = hierarchy.descendants();

  const nested = d3.nest()
    .key(colorValue)
    .entries(descendants);

  const xValue = d => d.values.length;
  const yValue = d => d.key;

  svg.append('g').call(visualList, {
    data: nested,
    xValue,
    yValue,
    colorValue: yValue,
    colorScale,
    width,
    margin: {
      left: 2,
      right: 20,
      top: 2,
      bottom: 20
    },
    barPadding: 0.1,
    barOpacity: 0.9,
    verticalSpacing: 17,
    transitionDuration: 800,
    textValue: d => `.${yValue(d)}  ${commaFormat(xValue(d))}`
  });


  var link = g.selectAll('.link')
      .data(descendants.slice(1))
    .enter().append('path')
      .attr('class', 'link')
      .attr('stroke', d => colorScale(colorValue(d)))
      .attr('d', function(d) {
        if(d.parent === descendants[0]){
          return 'M' + project(d.x, d.y)
            + ' ' + project(d.parent.x, d.parent.y);
        } else {
          return 'M' + project(d.x, d.y)
            + 'C' + project(d.x, (d.y + d.parent.y) / 2)
            + ' ' + project(d.parent.x, (d.y + d.parent.y) / 2)
            + ' ' + project(d.parent.x, d.parent.y);
        }
      });
});

function project(theta, r){
  theta += Math.PI/2;
  return [
    width / 2 + r * Math.sin(theta) + xOffset,
    height + r * Math.cos(theta) + yOffset
  ]
}